from imio.fpaudit import LOG_DIR
from imio.fpaudit import LOG_ENTRIES_REGISTRY
from imio.fpaudit.interfaces import ILogsStorage
from imio.fpaudit.logger import FPAuditLogInfo
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.component import queryUtility
from zope.interface import implementer

import os


def _build_storage(logs_config):
    """Build a dict of FPAuditLogInfo instances from a logs config list."""
    dic = {}
    for entry in logs_config:
        log_i = FPAuditLogInfo(
            {"audit-log": os.path.join(LOG_DIR, entry["audit_log"])},
            entry["log_id"],
            logformat=entry["log_format"],
        )
        log_i.handler.formatter.datefmt = "%Y-%m-%d %H:%M:%S"
        dic[entry["log_id"]] = log_i
    return dic


@implementer(ILogsStorage)
class LogsStorageUtility(object):
    """Utility to store logs instances"""

    def __init__(self):
        self.storage = {}
        self._loaded = False

    def _ensure_loaded(self):
        """Lazily populate storage from registry on first access."""
        if self._loaded:
            return
        registry = queryUtility(IRegistry)
        if registry is not None:
            log_entries = registry.get(LOG_ENTRIES_REGISTRY) or []
            if log_entries:
                self.storage = _build_storage(log_entries)
        self._loaded = True

    def add(self, key, value):
        """Add a log entry in storage"""
        self.storage[key] = value

    def set(self, dic):
        """Set storage to dic"""
        self.storage = dic
        self._loaded = True

    def get(self, key, default=None):
        """Get a key from storage"""
        self._ensure_loaded()
        return self.storage.get(key, default)

    def remove(self, key):
        """Remove key from storage"""
        if key in self.storage:
            del self.storage[key]


def store_config(logs_config):
    """Store logs configuration in utility."""
    storage = getUtility(ILogsStorage)
    storage.set(_build_storage(logs_config))



# -*- coding: utf-8 -*-

from imio.fpaudit.interfaces import IImioFPAuditLayer
from imio.fpaudit.testing import IMIO_FPAUDIT_FUNCTIONAL_TESTING
from imio.helpers import HAS_PLONE_6_AND_MORE
from plone.browserlayer import utils

import unittest


if HAS_PLONE_6_AND_MORE:
    from Products.CMFPlone.utils import get_installer


class TestSetup(unittest.TestCase):

    layer = IMIO_FPAUDIT_FUNCTIONAL_TESTING

    def setUp(self):
        portal = self.layer["portal"]
        if HAS_PLONE_6_AND_MORE:
            self.installer = get_installer(portal)

    def test_product_installed(self):
        """Test if imio.fpaudit is installed with portal_quickinstaller."""
        if HAS_PLONE_6_AND_MORE:
            self.assertTrue(self.installer.is_product_installed("imio.fpaudit"))

    def test_uninstall(self):
        """Test if imio.fpaudit is cleanly uninstalled."""
        if HAS_PLONE_6_AND_MORE:
            self.installer.uninstall_product("imio.fpaudit")
            self.assertFalse(self.installer.is_product_installed("imio.fpaudit"))

    def test_browserlayer(self):
        """Test that IImioFPAuditLayer is registered."""
        if HAS_PLONE_6_AND_MORE:
            self.assertIn(IImioFPAuditLayer, utils.registered_layers())
            self.installer.uninstall_product("imio.fpaudit")
            self.assertNotIn(IImioFPAuditLayer, utils.registered_layers())



# -*- coding: utf-8 -*-

from imio.fpaudit import LOG_ENTRIES_REGISTRY
from imio.fpaudit.utils import logger
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import INonInstallable
from ZODB.POSException import ConnectionStateError
from zope.component import getUtility
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            "imio.fpaudit:uninstall",
        ]

    def getNonInstallableProducts(self):
        """Hide the upgrades package from site-creation and quickinstaller."""
        return ["imio.fpaudit.upgrades"]


def post_install(context):
    """Post install script"""
    registry = getUtility(IRegistry)
    if registry[LOG_ENTRIES_REGISTRY] is None:
        # may fail in tests because a datagridfield is stored, just pass in this case
        try:
            registry[LOG_ENTRIES_REGISTRY] = []
        except ConnectionStateError:
            logger.warn("!!!Failed to set registry log_entries to []!!!")
            registry.records[LOG_ENTRIES_REGISTRY].field.value_type = None


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""

from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class IImioFPAuditLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class ILogsStorage(Interface):
    """Logs storage utility interface"""

    def add(key, value):
        """Add a log entry in storage"""

    def set(key, dic):
        """Set storage to dic"""

    def get(key, default=None):
        """Get a key from storage"""

    def remove(key):
        """Remove key from storage"""



# -*- coding: utf-8 -*-
"""Installer for the imio.fpaudit package."""

from setuptools import find_packages
from setuptools import setup


long_description = open("README.rst").read() + "\n\n" + open("CHANGES.rst").read() + "\n\n"


setup(
    name="imio.fpaudit",
    version="0.1.3",
    description="This package contains fingerpointing log audit helper.",
    long_description=long_description,
    # Get more from https://pypi.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: 6.1",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.13",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="Python Zope Plone",
    author="IMIO",
    author_email="dev@imio.be",
    url="http://pypi.python.org/pypi/imio.fpaudit",
    license="GPL V2",
    packages=find_packages("src", exclude=["ez_setup"]),
    namespace_packages=["imio"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "plone.api",
        "setuptools",
        "collective.fingerpointing",
        "collective.documentgenerator",
    ],
    extras_require={
        "test": [
            "plone.app.testing",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
    options={"bdist_wheel": {"universal": True}},
)



# -*- coding: utf-8 -*-
from imio.fpaudit import LOG_DIR
from imio.fpaudit import LOG_ENTRIES_REGISTRY
from imio.fpaudit.storage import store_config
from imio.fpaudit.testing import clear_temp_dir
from imio.fpaudit.testing import IMIO_FPAUDIT_INTEGRATION_TESTING
from imio.fpaudit.testing import write_temp_files
from imio.fpaudit.utils import fpalog
from imio.fpaudit.utils import get_all_lines_of
from imio.fpaudit.utils import get_lines_info
from imio.fpaudit.utils import get_lines_of
from imio.fpaudit.utils import get_logrotate_filenames
from plone import api

import os
import shutil
import tempfile
import unittest


class TestUtils(unittest.TestCase):

    layer = IMIO_FPAUDIT_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]

    def test_fpalog(self):
        api.portal.set_registry_record(
            LOG_ENTRIES_REGISTRY,
            [{"log_id": u"test", "audit_log": u"test_utils.log", "log_format": u"%(asctime)s - %(message)s"}],
        )
        log_file_path = os.path.join(LOG_DIR, "test_utils.log")
        for fil in get_logrotate_filenames(LOG_DIR, "test_utils.log", r".+$"):
            os.remove(fil)
        fpalog("test", "AUDIT", "extra 1")
        logs = get_logrotate_filenames(LOG_DIR, "test_utils.log", r"\.\d+$")
        self.assertListEqual(logs, [log_file_path])
        lines = [ln for ln in get_lines_of(log_file_path)]
        self.assertEqual(len(lines), 1)
        self.assertTrue(lines[0].endswith(" - user=test_user_1_ ip=None action=AUDIT extra 1"))
        fpalog("test", "AUDIT", "extra 2")
        lines = [ln for ln in get_lines_of(log_file_path)]
        self.assertEqual(len(lines), 2)
        self.assertTrue(lines[0].endswith(" - user=test_user_1_ ip=None action=AUDIT extra 2"))
        self.assertTrue(lines[1].endswith(" - user=test_user_1_ ip=None action=AUDIT extra 1"))
        # check with logrotated files
        log_file_path1 = log_file_path + ".1"
        os.rename(log_file_path, log_file_path1)
        # changed id to stop writing in rotated here
        store_config([{"log_id": u"test1", "audit_log": u"test_utils.log", "log_format": u"%(asctime)s - %(message)s"}])
        fpalog("test1", "AUDIT", "extra 3")
        fpalog("test1", "AUDIT", "extra 4")
        logs = get_logrotate_filenames(LOG_DIR, "test_utils.log", r"\.\d+$")
        lines = [ln for ln in get_all_lines_of(logs)]
        self.assertTrue(lines[0].endswith(" - user=test_user_1_ ip=None action=AUDIT extra 4"))
        self.assertTrue(lines[1].endswith(" - user=test_user_1_ ip=None action=AUDIT extra 3"))
        self.assertTrue(lines[2].endswith(" - user=test_user_1_ ip=None action=AUDIT extra 2"))
        self.assertTrue(lines[3].endswith(" - user=test_user_1_ ip=None action=AUDIT extra 1"))
        for fil in get_logrotate_filenames(LOG_DIR, "test_utils.log", r".+$"):
            os.remove(fil)

    def test_get_lines_info(self):
        line = "2024-10-10 14:59:07 - user=admin ip=127.0.0.1 action=AUDIT col_a=xx  xx col_b=yyy éé"
        dic = get_lines_info(line, ["col_a", "col_b"])
        self.assertDictEqual(
            dic,
            {
                "date": "2024-10-10 14:59:07",
                "user": "admin",
                "ip": "127.0.0.1",
                "action": "AUDIT",
                "col_a": "xx  xx",
                "col_b": "yyy éé",
            },
        )

    def test_get_lines_of(self):
        api.portal.set_registry_record(
            LOG_ENTRIES_REGISTRY,
            [{"log_id": u"test", "audit_log": u"test_utils.log", "log_format": u"%(asctime)s - %(message)s"}],
        )
        log_file_path = os.path.join(LOG_DIR, "test_utils.log")
        for fil in get_logrotate_filenames(LOG_DIR, "test_utils.log", r".+$"):
            os.remove(fil)
        fpalog("test", "AUDIT", "extra")
        fpalog("test", "CONTACTS", "extra")
        fpalog("test", "CONTACTS", "extra")
        fpalog("test", "AUDIT", "extra")
        fpalog("test", "GROUPS", "extra")
        lines = [ln for ln in get_lines_of(log_file_path)]
        self.assertEqual(len(lines), 5)
        lines = [ln for ln in get_lines_of(log_file_path, actions=("GROUPS",))]
        self.assertEqual(len(lines), 1)
        self.assertTrue(all("GROUPS" in ln for ln in lines))
        lines = [ln for ln in get_lines_of(log_file_path, actions=("AUDIT", "CONTACTS"))]
        self.assertEqual(len(lines), 4)
        self.assertEqual(len([ln for ln in lines if "AUDIT" in ln]), 2)
        self.assertEqual(len([ln for ln in lines if "CONTACTS" in ln]), 2)
        for fil in get_logrotate_filenames(LOG_DIR, "test_utils.log", r".+$"):
            os.remove(fil)

    def test_get_logrotate_filenames(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # check filter
            write_temp_files(temp_dir, ["test.log", "other.log", "test.log.1", "test.log.2", "test.log.lock"])
            expected_files = ["test.log", "test.log.1", "test.log.2"]
            result_files = get_logrotate_filenames(temp_dir, "test.log", r"\.\d+$", full=False)
            self.assertListEqual(result_files, expected_files)
            clear_temp_dir(temp_dir)
            # check order
            write_temp_files(temp_dir, ["test.log", "test.log.1", "test.log.2", "test.log.10"])
            expected_files = ["test.log", "test.log.1", "test.log.2", "test.log.10"]
            result_files = get_logrotate_filenames(temp_dir, "test.log", r"\.\d+$", full=False)
            self.assertListEqual(result_files, expected_files)
            clear_temp_dir(temp_dir)
            # check full path
            write_temp_files(temp_dir, ["test.log", "test.log.1", "test.log.2", "test.log.10"])
            expected_files = ["test.log", "test.log.1", "test.log.2", "test.log.10"]
            expected_files = [os.path.join(temp_dir, f) for f in expected_files]
            result_files = get_logrotate_filenames(temp_dir, "test.log", r"\.\d+$")
            self.assertListEqual(result_files, expected_files)
            clear_temp_dir(temp_dir)
            # checl another filter
            write_temp_files(
                temp_dir, ["test.log", "other.log", "test.log-20240825", "test.log-20240901", "test.log-20240908"]
            )
            expected_files = ["test.log", "test.log-20240825", "test.log-20240901", "test.log-20240908"]
            result_files = get_logrotate_filenames(temp_dir, "test.log", r"-\d{8}$", full=False)
            self.assertListEqual(result_files, expected_files)
        finally:
            shutil.rmtree(temp_dir)
